const MiniCssExtractPlugin = require("mini-css-extract-plugin");

module.exports = (mode, types, isModule, extract) => {
  function generateLoaders(loader, loaderOptions) {
    const cssLoader = {
      loader: 'css-loader',
      options: {}
    };

    if (isModule) {
      cssLoader.options.modules = true,
      cssLoader.options.localIdentName = '[path][name]__[local]--[hash:base64:5]';
    }

    const loaders = [cssLoader];
    
    if (loader) {
      loaders.push({
        loader: loader + '-loader',
        options: Object.assign({}, loaderOptions)
      });
    }

    if (extract && mode === 'production') {
      return [{ loader: MiniCssExtractPlugin.loader }].concat(loaders);
    } else {
      return [{ loader: 'style-loader' }].concat(loaders);
    }
  }

  const loaders = {
    css: generateLoaders(),
    postcss: generateLoaders(),
    less: generateLoaders('less', { javascriptEnabled: true }),
    sass: generateLoaders('sass', { indentedSyntax: true }),
    scss: generateLoaders('sass'),
    stylus: generateLoaders('stylus'),
    styl: generateLoaders('stylus')
  }

  types = types.concat(['css']);

  const output = [];

  for (const extension of types) {
    const loader = loaders[extension];
    output.push({
      test: new RegExp('\\.' + extension + '$'),
      use: loader,
      exclude: /\.module\.[a-z]+$/
    });
  }

  if (isModule) {
    for (const extension of types) {
      const loader = loaders[extension];
      output.push({
        test: new RegExp('\\.module\\.' + extension + '$'),
        use: loader,
        include: /\.module\.[a-z]+$/
      });
    }
  }

  return output;
}
